/* property.c
 * Copyright (C) 2002-2003 Pascal Eberhard <pascal.ebo@netcourrier.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "common.h"

static void property_refresh_app(property_t *prop);
static void property_refresh_no_app(property_t *prop);

static void property_build_interface(property_t *prop);
static void property_refreshed(Cereimg *cell, gchar *path_str, gchar *new_text,
                            property_t *prop);
static void cereimg_clicked(Cereimg *cell, gchar *path_str,
                            property_t *prop);
static void property_remove_clicked(GtkWidget *button, property_t *prop);

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
property_t *property_new(apwalapp_t *apwal)
{
  property_t        *prop;

  g_assert(apwal != NULL);
  g_assert(apwal->property_frame != NULL);

  prop = (property_t*) malloc(sizeof(property_t));
  g_assert(prop != NULL);

  prop->apwal = apwal;
  property_build_interface(prop);
  property_refresh_no_app(prop);

  return prop;
}
// ----------------------------------------------------------------------------
void property_refresh(property_t *prop)
{
  g_assert(prop != NULL);
  g_assert(prop->apwal != NULL);

  if (prop->apwal->selected_app != NULL)
    property_refresh_app(prop);
  else
    property_refresh_no_app(prop);
}
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void property_refresh_app(property_t *prop)
{
  GtkTreeIter  iter;
  gchar       *old_val;
  //gboolean    iter_is_ok;
  GdkPixbuf *pixbuf_big;
  GdkPixbuf *pixbuf_sml;
  app_t     *app;

  g_assert(prop != NULL);
  g_assert(prop->apwal != NULL);
  g_assert(prop->apwal->apps != NULL);
  g_assert(prop->apwal->selected_app != NULL);

  TRACE("app:%p", prop->apwal->selected_app);

  app = prop->apwal->selected_app;

  // data on model has not to be freed, it's done by itself
  gtk_tree_model_get_iter_first(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, app->name, -1);

  
  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, app->exec, -1);

  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, app->args, -1);

  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, app->path, -1);

  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, app->icon, -1);

  pixbuf_sml = gdk_pixbuf_new_from_apwal("relief", NULL, NULL);
  pixbuf_big = gdk_pixbuf_new_from_apwal(app->icon, NULL, NULL);
  gdk_pixbuf_append(pixbuf_sml, pixbuf_big);
//pixbuf_sml = gdk_pixbuf_scale_simple(pixbuf_big, 16, 16, GDK_INTERP_BILINEAR);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 3, pixbuf_sml, -1);
  g_object_unref(pixbuf_big);
  g_object_unref(pixbuf_sml);

  gtk_widget_set_sensitive(prop->treeview, TRUE); 
  gtk_widget_set_sensitive(prop->btn_remove, TRUE); 
}
// ----------------------------------------------------------------------------
static void property_refresh_no_app(property_t *prop)
{
  GtkTreeIter  iter;
  gchar       *old_val;

  g_assert(prop != NULL);
  g_assert(prop->apwal != NULL);
  g_assert(prop->apwal->selected_app == NULL);

  TRACE("%s", "");

  prop->apwal->selected_app_field = -1;

  gtk_widget_set_sensitive(prop->treeview, FALSE); 
  gtk_widget_set_sensitive(prop->btn_remove, FALSE); 

  gtk_tree_model_get_iter_first(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, ""/*name*/, -1);

  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, ""/*exec*/, -1);

  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, ""/*args*/, -1);

  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, ""/*path*/, -1);

  gtk_tree_model_iter_next(prop->model, &iter);
  gtk_tree_model_get(prop->model, &iter, 1, &old_val, -1);
  g_free(old_val);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, ""/*icon*/, -1);
  gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter,
                     3, NULL, -1);
}
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void property_build_interface(property_t *prop)
{
//GtkWidget         *property_frame;
  GtkWidget           *hbox;
  GtkWidget             *vbox_tv;
//GtkWidget               *treeview;
  GtkTreeStore              *store;
  GtkTreeIter               iter;
  GtkTreeSelection         *select;
  GtkTreeViewColumn        *column;
  GtkCellRenderer            *renderer;
  GtkWidget           *vbox;
//GtkWidget             *btn_remove;
  GdkPixbuf *pix1;

  g_assert(prop != NULL);
  g_assert(prop->apwal != NULL);
  g_assert(prop->apwal->property_frame != NULL);

  // horizontal box
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
  gtk_container_add(GTK_CONTAINER(prop->apwal->property_frame), hbox);
  gtk_widget_show(hbox);

  vbox_tv = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox_tv), 2);
  gtk_box_pack_start(GTK_BOX(hbox), vbox_tv, TRUE, TRUE, 0);
  gtk_widget_show(vbox_tv);

  //prop->pixbuf_open = gtk_widget_render_icon(prop->apwal->window, "gtk-open", 
  //                                           GTK_ICON_SIZE_MENU, NULL);
  prop->pixbuf_open = gdk_pixbuf_new_from_apwal("relief", NULL, NULL);
  pix1 = gtk_widget_render_icon(prop->apwal->window, "gtk-open",
                                GTK_ICON_SIZE_MENU, NULL);
  gdk_pixbuf_append(prop->pixbuf_open, pix1);

  store = gtk_tree_store_new(4, G_TYPE_STRING, G_TYPE_STRING,
                                G_TYPE_STRING, GDK_TYPE_PIXBUF);
  gtk_tree_store_append(store, &iter, NULL);
  gtk_tree_store_set(store, &iter,
                     0, "Name",
                     1, "",
                     2, "#ddddff",
                     3, NULL,
                     -1);
  gtk_tree_store_append(store, &iter, NULL);
  gtk_tree_store_set(store, &iter,
                     0, "Prog",
                     1, "",
                     2, "#eeeeff",
                     3, prop->pixbuf_open,
                     -1);
  gtk_tree_store_append(store, &iter, NULL);
  gtk_tree_store_set(store, &iter,
                     0, "Args",
                     1, "",
                     2, "#ddddff",
                     3, NULL,
                     -1);
  gtk_tree_store_append(store, &iter, NULL);
  gtk_tree_store_set(store, &iter,
                     0, "Path",
                     1, "",
                     2, "#eeeeff",
                     3, prop->pixbuf_open,
                     -1);
  gtk_tree_store_append(store, &iter, NULL);
  gtk_tree_store_set(store, &iter,
                     0, "Icon",
                     1, "",
                     2, "#ddddff",
                     3, prop->pixbuf_open,
                     -1);

  prop->treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
  g_object_unref(G_OBJECT(store));
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(prop->treeview), FALSE);
  prop->model = gtk_tree_view_get_model(GTK_TREE_VIEW(prop->treeview));

  // pixmap
  renderer = cereimg_new();
  g_object_set(renderer, "activatable", TRUE,
                         "background", "#ccccff",
                         NULL);

  g_signal_connect(G_OBJECT(renderer), "clicked",
                   G_CALLBACK(cereimg_clicked), prop);
  column = gtk_tree_view_column_new();
  gtk_tree_view_column_pack_start(column, renderer, FALSE/*expand*/);
  gtk_tree_view_column_set_attributes(column, renderer,
                                      "pixbuf",  3,
                                      NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(prop->treeview), column);
    
  // type 
  renderer = gtk_cell_renderer_text_new();
  g_object_set(G_OBJECT(renderer), "background", "#ccccff", NULL);
  column = gtk_tree_view_column_new_with_attributes(
             "Attribut", renderer, "text", 0,
             NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(prop->treeview), column);

  // value
  renderer = gtk_cell_renderer_text_new();
  g_object_set(renderer, "editable", TRUE, NULL);
  g_signal_connect(G_OBJECT(renderer), "edited",
                   G_CALLBACK(property_refreshed), prop);
  column = gtk_tree_view_column_new();
  gtk_tree_view_column_set_title(column, "Value");
  //gtk_tree_view_column_set_min_width(column, 200);
  //gtk_tree_view_column_set_max_width(column, 200);
  gtk_tree_view_column_pack_start(column, renderer, TRUE);
  gtk_tree_view_column_set_attributes(column, renderer,
                                      "text", 1, 
                                      "background", 2,
                                      NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(prop->treeview), column);

  // selection
  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(prop->treeview));
  gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

  gtk_box_pack_start(GTK_BOX(vbox_tv), prop->treeview, TRUE, TRUE, 0);
  gtk_widget_show(prop->treeview);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 2);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);

  prop->btn_remove = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
  g_signal_connect(G_OBJECT(prop->btn_remove), "clicked",
                   G_CALLBACK(property_remove_clicked), prop);
  gtk_box_pack_end(GTK_BOX(vbox), prop->btn_remove, FALSE, FALSE, 0);
  gtk_tooltips_set_tip(prop->apwal->tips, prop->btn_remove, "Remove\nRemove the application currently selected in the editor", NULL);
  gtk_widget_show(prop->btn_remove);

}
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void property_refreshed(Cereimg *cell, gchar *path_str, gchar *new_text,
                            property_t *prop)
{
  GtkTreePath  *path;
  GtkTreeIter   iter;
  gchar       **app_var;
//gchar        *app_var_from_model;

  g_assert(prop != NULL);
  g_assert(prop->apwal != NULL);
  g_assert(prop->apwal->selected_app != NULL);

  TRACE("path_str:%s, new_text:%s", path_str, new_text);
  
  // position iter on the edited row
  path = gtk_tree_path_new_from_string(path_str);
  gtk_tree_model_get_iter(prop->model, &iter, path);
  gtk_tree_path_free(path);

  // select the property edited on app structure
  switch(path_str[0])     /* first row = '0' = 48 = name */
  {
  case 48+0: app_var = & prop->apwal->selected_app->name; break;
  case 48+1: app_var = & prop->apwal->selected_app->exec; break;
  case 48+2: app_var = & prop->apwal->selected_app->args; break;
  case 48+3: app_var = & prop->apwal->selected_app->path; break;
  case 48+4: app_var = & prop->apwal->selected_app->icon; break;
  default:
    ERR("path_str:%s, [0]:%d, new_text:%s", path_str, path_str[0], new_text);
  }

  if (*app_var)
    g_free(*app_var);
  *app_var = g_strdup(new_text);

  //gtk_tree_model_get(prop->model, &iter, 1, &app_var_from_model, -1);
  //g_free(app_var_from_model);
  //gtk_tree_store_set(GTK_TREE_STORE(prop->model), &iter, 1, *app_var, -1);
  apwalapp_selected_app_modified(prop->apwal);
}
// ----------------------------------------------------------------------------
void prop_compute_new_path(gchar *olddir, gchar *newdir, void *data)
{
  property_t *prop = (property_t *) data;
  g_assert(prop != NULL);
  TRACE("newdir:%s", newdir);
  if (newdir != NULL)
  {
    app_field_set_value(prop->apwal->selected_app,
                        prop->apwal->selected_app_field,
                        newdir);
    apwalapp_selected_app_modified(prop->apwal);
  }
}

// ----------------------------------------------------------------------------
static void cereimg_clicked(Cereimg *cell, gchar *path_str,
                            property_t *prop)
{
  GtkTreePath  *path;
  gint *indices;
  gchar *olddir;

  g_assert((cell != NULL) && (path_str != NULL) && (prop != NULL));
  g_assert(prop->apwal != NULL);
  g_assert(prop->apwal->apps != NULL);
  g_assert(prop->apwal->selected_app != NULL);

  path = gtk_tree_path_new_from_string(path_str);
  gtk_tree_model_get_iter(prop->model, &prop->selected_iter, path);
  indices = gtk_tree_path_get_indices(path);
  g_assert(indices != NULL);
  prop->apwal->selected_app_field = indices[0];
  gtk_tree_path_free(path);

  //gtk_tree_model_get(prop->model, &iter, 0, &dirname, 1, &selected, -1);
  switch(prop->apwal->selected_app_field)
  {
  case APP_FIELD_EXEC:
    filesel_edit(prop->apwal->filesel);
    break;
  case APP_FIELD_PATH:
    olddir = app_field_get_value(prop->apwal->selected_app,
                                 prop->apwal->selected_app_field);
    dirsel_select(prop->apwal->dirsel, olddir, prop_compute_new_path, prop);
    break;
  case APP_FIELD_ICON:
    apwalapp_goto_iconsel(prop->apwal);
    break;
  case 0: /* name */
  case 2: /* args */
    break;
  default:
    ERR("row:%d", prop->apwal->selected_app_field);
    break;
  }
}

// ----------------------------------------------------------------------------
static void property_remove_clicked(GtkWidget *button, property_t *prop)
{
  app_t *app;
  g_assert((button != NULL) && (prop != NULL));
  g_assert(prop->apwal != NULL);
  g_assert(prop->apwal->selected_app != NULL);

  TRACE("app:%p", prop->apwal->selected_app);
  app = prop->apwal->selected_app;
  app_list_remove(prop->apwal->apps, app);
  prop->apwal->selected_app = NULL;

  apwalapp_selected_app_modified(prop->apwal);
}
// ----------------------------------------------------------------------------
