/* sysstuff.c
 * Copyright (C) 2002-2003 Pascal Eberhard <pascal.ebo@netcourrier.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "log.h"
#include <stdlib.h>
#include <errno.h>
#include <gtk/gtk.h>

#include <stdio.h>      // FILENAME_MAX
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#define  __USE_GNU 1
#include <unistd.h>     // group_member, chdir

//#ifndef __USE_GNU /* unistd.h */
//extern int group_member(gid_t gid);
//extern char *get_current_dir_name(void);
//#endif

// ----------------------------------------------------------------------------
gboolean sys_can_access(gchar *filename)
{
  struct stat statbuf;
  gint cc;
  gboolean is_read_access;

  is_read_access = FALSE;
  cc = stat(filename, &statbuf);
  if (cc == 0)
  {
    // check if the file is readable for the current user
    //TRACE("mode:%x mode:%x, res:%x",
    //       statbuf.st_mode, S_IROTH, (statbuf.st_mode & S_IROTH));
    if ((statbuf.st_mode & S_IROTH) == S_IROTH)
    {
      is_read_access = TRUE;
    }
    if (((statbuf.st_mode & S_IRGRP) == S_IRGRP) &&
        ((statbuf.st_gid == getegid()) || group_member(statbuf.st_gid)))
      is_read_access = TRUE;
    if (((statbuf.st_mode & S_IRUSR) == S_IRUSR) &&
         (statbuf.st_uid == geteuid()))
      is_read_access = TRUE;
  }
/*
  else
  {
    // file not accessible
    int er;
    er = errno;
    switch(er)
    {
    case ENOENT:
    case ENOTDIR:
      TRACE("stat, exec filename:%s, e:%d, file does not exist",
              filename, er);
      break;
    case EACCES:
      TRACE("stat, exec filename:%s, e:%d, permission denied",
              filename, er);
      break;
    default:  
      TRACE("stat, exec filename:%s, e:%d", filename, er);
    }
  }
*/
  return is_read_access;
}
// ----------------------------------------------------------------------------
static gboolean sys_have_exec_access(gchar *param_filename)
{
  struct stat statbuf;
  gint cc;
  gboolean found;
  gchar *filename;
  gchar *firstspace;

  g_assert(param_filename != NULL);
  
  // take only the executable file without parameters
  firstspace = param_filename;
  while(firstspace[0] != '\0' && firstspace[0] != ' ')
  {
    firstspace++;
  }
  filename = g_strndup(param_filename, (firstspace - param_filename));

  found = FALSE;
  cc = stat(filename, &statbuf);
  if (cc == 0)
  {
    // check if the file is executable for the current user
    //TRACE("mode:%x mode:%x, res:%x",
    //       statbuf.st_mode, S_IXOTH, (statbuf.st_mode & S_IXOTH));
    if ((statbuf.st_mode & S_IXOTH) == S_IXOTH)
    {
      found = TRUE;
    }
    if (((statbuf.st_mode & S_IXGRP) == S_IXGRP) &&
        ((statbuf.st_gid == getegid()) || group_member(statbuf.st_gid)))
      found = TRUE;
    if (((statbuf.st_mode & S_IXUSR) == S_IXUSR) &&
         (statbuf.st_uid == geteuid()))
      found = TRUE;
  }
/* verbose
  else
  {
    // file not accessible
    int er;
    er = errno;
    switch(er)
    {
    case ENOENT:
    case ENOTDIR:
      TRACE("stat, exec filename:%s, e:%d, file does not exist",
              filename, er);
      break;
    case EACCES:
      TRACE("stat, exec filename:%s, e:%d, permission denied",
              filename, er);
      break;
    default:  
      TRACE("stat, exec filename:%s, e:%d", filename, er);
    }
  }
*/

  g_free(filename);
  return found;
}

// ----------------------------------------------------------------------------
gboolean sys_can_exec(gchar *cmdline)
{
  gchar *path, *cur, *cur_last, tmp[FILENAME_MAX];
  gboolean found;

  g_assert(cmdline != NULL);

  if (sys_have_exec_access(cmdline))
    return TRUE;
    
  path = getenv("PATH");
  if (!path)
  { WARN("%s", "getenv, PATH not found");
    return FALSE;
  }

  // create the list of all the possibility of path combination for
  // the executable file.
  path = strdup(path);
  cur = cur_last = path;
  found = FALSE;
  while( (cur = strstr(cur_last,":")) && (found == FALSE))
  {
    cur[0] = '\0';
    strcpy(tmp, cur_last);
    strcat(tmp, "/");
    strcat(tmp, cmdline);
    //TRACE("tmp:%s",tmp);

    if (sys_have_exec_access(tmp))
      found = TRUE;

    cur++;
    cur_last = cur;
  }
  g_free(path);
  return found;
}
// ----------------------------------------------------------------------------
gboolean sys_exec(gchar *path, gchar *prog, gchar *args)
{
  int cc;
  int args_len;
  gboolean  spawned;
  GError   *err = NULL;
  gchar    *cmdline1;
  gchar    *cmdline2;
  const gchar *homedir;
  gchar    *currdir;
  
  g_assert(prog != NULL);

  currdir = g_get_current_dir();
  g_assert(currdir != NULL);

  if (path == NULL)
    homedir = g_get_home_dir();
  else if (strcmp(path, "") == 0)
    homedir = g_get_home_dir();
  else
    homedir = NULL;

  if (homedir == NULL)
    cc = chdir(path);
  else
    cc = chdir(homedir);
  if (cc)
    WARN("chdir:%d, errno:%d, path:%s, homedir:%s", cc, errno, path, homedir);

  args_len = 0;
  if (args != NULL)
  {
    args_len = strlen(args);
    if (args_len > 0)
      args_len += 1/*because prog" "args*/;
  }
  cmdline1 = (gchar *) malloc(strlen(prog) + args_len);
  strcpy(cmdline1, prog);
  if (args_len > 0)
  {
    strcat(cmdline1, " ");
    strcat(cmdline1, args);
  }

  // try launch the exec if it is in the path
  spawned = g_spawn_command_line_async(cmdline1, &err/*GError*/);
  if (spawned)
  {
    cc = chdir(currdir);
    if (cc)
      WARN("chdir:%d, errno:%d, currdir:%s", cc, errno, currdir);
    free(cmdline1);
    return TRUE;
  }

  g_assert(err != NULL && err->message != NULL);
  WARN("g_spawn_command_line_async, cmdline1:%s, E:%d, %s",
       cmdline1, err->code, err->message);
  g_error_free(err);
  err = NULL;

  // try lanch the exec if it is in the local directory
  cmdline2 = (gchar*) malloc(strlen(cmdline1) + 2/*'./'*/ + 1/*'\0'*/);
  strcpy(cmdline2, "./");
  strcat(cmdline2, cmdline1);
  spawned = g_spawn_command_line_async(cmdline2, &err/*GError*/);
  if (spawned)
  {
    cc = chdir(currdir);
    if (cc)
      WARN("chdir:%d, errno:%d, currdir:%s", cc, errno, currdir);
    free(cmdline1);
    free(cmdline2);
    return TRUE;
  }

  g_assert(err != NULL && err->message != NULL);
  WARN("g_spawn_command_line_async, cmdline2:%s, E:%d, %s",
            cmdline2, err->code, err->message);
  free(cmdline1);
  free(cmdline2);
  g_error_free(err);
  err = NULL;
  return FALSE;
}
/* obsolete
  if (fork() == 0)
  {
    int cc;
    // child
    TRACE("app, exec:%s", app->exec);
    cc = execlp(app->exec, app->exec, NULL);
    if (cc)
    {
      TRACE("ERROR: execvp %s, E:%d", app->exec, errno);
      exit(0);
    }
  }
*/
// ----------------------------------------------------------------------------
gboolean sys_file_exist(char *file)
{
  struct stat statbuf;
  int cc;
  if (file == NULL)
    return FALSE;
  cc = stat(file, &statbuf);
  if (cc)
  {
    return FALSE;
  }
  return TRUE;
}
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
char * sys_filename(char *file)         /* oups, i didn't saw man 3 basename */
{
  static char filename[FILENAME_MAX];
  char *ptr;
  g_assert(file);
  g_assert(file != filename);
  ptr = file + strlen(file);
  if (ptr == file)
  {
    strcpy(filename, ".");
    return filename;
  }
  while(ptr != file && *ptr != '/')
    ptr--;

  if (*ptr == '/')
    strcpy(filename, ptr+1);
  else
    strcpy(filename, file);

  return filename;
}
// ----------------------------------------------------------------------------
char * sys_dirname(char *file)          /* oups, i didn't saw man 3 basename */
{
  static char dirname[FILENAME_MAX];
  char *ptr;
  g_assert(file);
  g_assert(file != dirname);
  ptr = file + strlen(file);
  while(ptr != file && *ptr != '/')
    ptr--;

  if (ptr == file && *ptr == '/')
      strcpy(dirname, "/");
  else if (ptr == file)
    strcpy(dirname, ".");
  else
  {
    strncpy(dirname, file, ptr - file);
    dirname[ptr - file] = '\0';
  }
  return dirname;
}

// ----------------------------------------------------------------------------
// the objectif is to launch the editor using the SAME binary file that the
// launcher. This can avoid problem like having another version somewhere of
// apwal or simply apwal could not be in the binary path.
// FIXME: sys_launch_editor : is a better way than using /proc filesystem?
void sys_launch_editor(void)
{
  static char procfile[FILENAME_MAX];
  static char execfile[FILENAME_MAX];
  int cc;
  int pid;
  int len;
  gboolean spawned;
  //FIXME:
  //getpid

  // build procfile, the link in /proc to the executable : /proc/'pid'/exe
  strcpy(procfile, "/proc/");
  len = strlen(procfile);
  pid = getpid();
  sprintf(procfile + len, "%d", pid);
  strcat(procfile, "/exe");

  // read procfile to get the binary file of apwal
  cc = readlink(procfile, execfile, FILENAME_MAX);
  if (cc == FILENAME_MAX)
    execfile[FILENAME_MAX-1] = '\0';
  else if (cc < 0)
    ERR("readlink(%s, execfile, %d):%d, errno:%d",
         procfile, FILENAME_MAX, cc, errno);

  TRACE("apwal binary file: %s", execfile);
  spawned = sys_exec(NULL, execfile, "--edit");
  if (!spawned)
  {
    WARN("sys_exec, '%s --edit'", execfile);
  }
}


// ----------------------------------------------------------------------------
// remove all not necessary '/' in the path
void make_path_uniq(char **path)
{
  int i, len, dbl_slash_cnt, dbl_slash_cnt_old;
  char *new_path;
  len = strlen(*path);
  dbl_slash_cnt = 0;
  for (i=0; i<len; i++)
  {
    if ( (*path)[i] == '/' )
    {
      if ( (i != 0) && ((*path)[i-1] == '/') )
        dbl_slash_cnt++;
      if ( ( i != 0) && (i == (len-1)) )
        dbl_slash_cnt++;
    }
  }
  if (dbl_slash_cnt == 0)
    return;

  new_path = (char*) g_malloc((len - dbl_slash_cnt + 1) * sizeof(char));
  dbl_slash_cnt = 0;
  dbl_slash_cnt_old = 0;
  for (i=0; i<len; i++)
  {
    if ( (*path)[i] == '/' )
    {
      if ( (i != 0) && ((*path)[i-1] == '/') )
        dbl_slash_cnt++;
      if ( (i != 0) && (i == (len-1)) )
        dbl_slash_cnt++;
    }
    if (dbl_slash_cnt_old == dbl_slash_cnt)
      new_path[i - dbl_slash_cnt] = (*path)[i];
    dbl_slash_cnt_old = dbl_slash_cnt;
  }
  new_path[len - dbl_slash_cnt] = '\0';
  g_free(*path);
  *path = new_path;
}
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
